@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printast.ml,v 1.28.4.1 2005/11/16 16:01:12 doligez Exp $ *)
+(* $Id: printast.ml 10263 2010-04-17 14:45:12Z garrigue $ *)
 
 open Asttypes;;
 open Format;;
@@ -65,6 +65,12 @@ let fmt_virtual_flag f x =
   | Concrete -> fprintf f "Concrete";
 ;;
 
+let fmt_override_flag f x =
+  match x with
+  | Override -> fprintf f "Override";
+  | Fresh -> fprintf f "Fresh";
+;;
+
 let fmt_rec_flag f x =
   match x with
   | Nonrecursive -> fprintf f "Nonrec";
@@ -89,7 +95,14 @@ let line i f s (*...*) =
   fprintf f s (*...*)
 ;;
 
-let list i f ppf l = List.iter (f i ppf) l;;
+let list i f ppf l =
+  match l with
+  | [] -> line i ppf "[]\n";
+  | h::t ->
+     line i ppf "[\n";
+     List.iter (f (i+1) ppf) l;
+     line i ppf "]\n";
+;;
 
 let option i f ppf x =
   match x with
@@ -139,6 +152,13 @@ let rec core_type i ppf x =
       line i ppf "Ptyp_poly%a\n"
         (fun ppf -> List.iter (fun x -> fprintf ppf " '%s" x)) sl;
       core_type i ppf ct;
+  | Ptyp_package (s, l) ->
+      line i ppf "Ptyp_package %a\n" fmt_longident s;
+      list i package_with ppf l
+
+and package_with i ppf (s, t) =
+  line i ppf "with type %s\n" s;
+  core_type i ppf t
 
 and core_field_type i ppf x =
   line i ppf "core_field_type %a\n" fmt_location x.pfield_loc;
@@ -169,7 +189,7 @@ and pattern i ppf x =
   | Ppat_variant (l, po) ->
       line i ppf "Ppat_variant \"%s\"\n" l;
       option i pattern ppf po;
-  | Ppat_record (l) ->
+  | Ppat_record (l, c) ->
       line i ppf "Ppat_record\n";
       list i longident_x_pattern ppf l;
   | Ppat_array (l) ->
@@ -179,12 +199,15 @@ and pattern i ppf x =
       line i ppf "Ppat_or\n";
       pattern i ppf p1;
       pattern i ppf p2;
+  | Ppat_lazy p ->
+      line i ppf "Ppat_lazy\n";
+      pattern i ppf p;
   | Ppat_constraint (p, ct) ->
       line i ppf "Ppat_constraint";
       pattern i ppf p;
       core_type i ppf ct;
   | Ppat_type li ->
-      line i ppf "PPat_type";
+      line i ppf "Ppat_type";
       longident i ppf li
 
 and expression i ppf x =
@@ -295,6 +318,16 @@ and expression i ppf x =
   | Pexp_object s ->
       line i ppf "Pexp_object";
       class_structure i ppf s
+  | Pexp_newtype (s, e) ->
+      line i ppf "Pexp_newtype \"%s\"\n" s;
+      expression i ppf e
+  | Pexp_pack (me, (p,l)) ->
+      line i ppf "Pexp_pack %a" fmt_longident p;
+      list i package_with ppf l;
+      module_expr i ppf me
+  | Pexp_open (m, e) ->
+      line i ppf "Pexp_open \"%a\"\n" fmt_longident m;
+      expression i ppf e
 
 and value_description i ppf x =
   line i ppf "value_description\n";
@@ -310,6 +343,7 @@ and type_declaration i ppf x =
   list (i+1) core_type_x_core_type_x_location ppf x.ptype_cstrs;
   line i ppf "ptype_kind =\n";
   type_kind (i+1) ppf x.ptype_kind;
+  line i ppf "ptype_private = %a\n" fmt_private_flag x.ptype_private;
   line i ppf "ptype_manifest =\n";
   option (i+1) core_type ppf x.ptype_manifest;
 
@@ -317,14 +351,12 @@ and type_kind i ppf x =
   match x with
   | Ptype_abstract ->
       line i ppf "Ptype_abstract\n"
-  | Ptype_variant (l, priv) ->
-      line i ppf "Ptype_variant %a\n" fmt_private_flag priv;
+  | Ptype_variant l ->
+      line i ppf "Ptype_variant\n";
       list (i+1) string_x_core_type_list_x_location ppf l;
-  | Ptype_record (l, priv) ->
-      line i ppf "Ptype_record %a\n" fmt_private_flag priv;
+  | Ptype_record l ->
+      line i ppf "Ptype_record\n";
       list (i+1) string_x_mutable_flag_x_core_type_x_location ppf l;
-  | Ptype_private ->
-      line i ppf "Ptype_private\n"
 
 and exception_declaration i ppf x = list i core_type ppf x
 
@@ -353,16 +385,19 @@ and class_type_field i ppf x =
   | Pctf_inher (ct) ->
       line i ppf "Pctf_inher\n";
       class_type i ppf ct;
-  | Pctf_val (s, mf, cto, loc) ->
+  | Pctf_val (s, mf, vf, ct, loc) ->
       line i ppf
-        "Pctf_val \"%s\" %a %a\n" s fmt_mutable_flag mf fmt_location loc;
-      option i core_type ppf cto;
+        "Pctf_val \"%s\" %a %a %a\n" s
+        fmt_mutable_flag mf fmt_virtual_flag vf fmt_location loc;
+      core_type (i+1) ppf ct;
   | Pctf_virt (s, pf, ct, loc) ->
       line i ppf
         "Pctf_virt \"%s\" %a %a\n" s fmt_private_flag pf fmt_location loc;
+      core_type (i+1) ppf ct;
   | Pctf_meth (s, pf, ct, loc) ->
       line i ppf
         "Pctf_meth \"%s\" %a %a\n" s fmt_private_flag pf fmt_location loc;
+      core_type (i+1) ppf ct;
   | Pctf_cstr (ct1, ct2, loc) ->
       line i ppf "Pctf_cstr %a\n" fmt_location loc;
       core_type i ppf ct1;
@@ -424,21 +459,25 @@ and class_structure i ppf (p, l) =
 
 and class_field i ppf x =
   match x with
-  | Pcf_inher (ce, so) ->
-      line i ppf "Pcf_inher\n";
+  | Pcf_inher (ovf, ce, so) ->
+      line i ppf "Pcf_inher %a\n" fmt_override_flag ovf;
       class_expr (i+1) ppf ce;
       option (i+1) string ppf so;
-  | Pcf_val (s, mf, e, loc) ->
-      line i ppf
-        "Pcf_val \"%s\" %a %a\n" s fmt_mutable_flag mf fmt_location loc;
+  | Pcf_valvirt (s, mf, ct, loc) ->
+      line i ppf "Pcf_valvirt \"%s\" %a %a\n"
+        s fmt_mutable_flag mf fmt_location loc;
+      core_type (i+1) ppf ct;
+  | Pcf_val (s, mf, ovf, e, loc) ->
+      line i ppf "Pcf_val \"%s\" %a %a %a\n"
+        s fmt_mutable_flag mf fmt_override_flag ovf fmt_location loc;
       expression (i+1) ppf e;
   | Pcf_virt (s, pf, ct, loc) ->
-      line i ppf
-        "Pcf_virt \"%s\" %a %a\n" s fmt_private_flag pf fmt_location loc;
+      line i ppf "Pcf_virt \"%s\" %a %a\n"
+        s fmt_private_flag pf fmt_location loc;
       core_type (i+1) ppf ct;
-  | Pcf_meth (s, pf, e, loc) ->
-      line i ppf
-        "Pcf_meth \"%s\" %a %a\n" s fmt_private_flag pf fmt_location loc;
+  | Pcf_meth (s, pf, ovf, e, loc) ->
+      line i ppf "Pcf_meth \"%s\" %a %a %a\n"
+        s fmt_private_flag pf fmt_override_flag ovf fmt_location loc;
       expression (i+1) ppf e;
   | Pcf_cstr (ct1, ct2, loc) ->
       line i ppf "Pcf_cstr %a\n" fmt_location loc;
@@ -477,6 +516,9 @@ and module_type i ppf x =
       line i ppf "Pmty_with\n";
       module_type i ppf mt;
       list i longident_x_with_constraint ppf l;
+  | Pmty_typeof m ->
+      line i ppf "Pmty_typeof\n";
+      module_expr i ppf m
 
 and signature i ppf x = list i signature_item ppf x
 
@@ -525,7 +567,11 @@ and with_constraint i ppf x =
   | Pwith_type (td) ->
       line i ppf "Pwith_type\n";
       type_declaration (i+1) ppf td;
+  | Pwith_typesubst (td) ->
+      line i ppf "Pwith_typesubst\n";
+      type_declaration (i+1) ppf td;
   | Pwith_module (li) -> line i ppf "Pwith_module %a\n" fmt_longident li;
+  | Pwith_modsubst (li) -> line i ppf "Pwith_modsubst %a\n" fmt_longident li;
 
 and module_expr i ppf x =
   line i ppf "module_expr %a\n" fmt_location x.pmod_loc;
@@ -547,6 +593,10 @@ and module_expr i ppf x =
       line i ppf "Pmod_constraint\n";
       module_expr i ppf me;
       module_type i ppf mt;
+  | Pmod_unpack (e, (p, l)) ->
+      line i ppf "Pmod_unpack %a\n" fmt_longident p;
+      list i package_with ppf l;
+      expression i ppf e;
 
 and structure i ppf x = list i structure_item ppf x
 
@@ -614,11 +664,11 @@ and core_type_x_core_type_x_location i ppf (ct1, ct2, l) =
   core_type (i+1) ppf ct2;
 
 and string_x_core_type_list_x_location i ppf (s, l, loc) =
-  string i ppf s;
+  line i ppf "\"%s\" %a\n" s fmt_location loc;
   list (i+1) core_type ppf l;
 
 and string_x_mutable_flag_x_core_type_x_location i ppf (s, mf, ct, loc) =
-  line i ppf "\"%s\" %a\n" s fmt_mutable_flag mf;
+  line i ppf "\"%s\" %a %a\n" s fmt_mutable_flag mf fmt_location loc;
   core_type (i+1) ppf ct;
 
 and string_list_x_location i ppf (l, loc) =
